ExGram.FSM.Filter.InFlow (ExGram FSM v0.1.0)

Copy Markdown View Source

An ExGram.Router.Filter that matches when any FSM flow is active.

This filter is automatically registered as the :in_flow alias when use ExGram.FSM detects that use ExGram.Router has also been called on the same module.

Usage

Use in a scope to match when any flow is active:

scope do
  filter :in_flow
  handle &MyBot.Handlers.in_flow/1
end